package xyz.lychee.lagfixer.libs.adventure.text.minimessage.internal.serializer;

import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.function.Predicate;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

class StyleClaimImpl<V> implements StyleClaim<V> {
   private final String claimKey;
   private final Function<Style, V> lens;
   private final Predicate<V> filter;
   private final BiConsumer<V, TokenEmitter> emitable;

   StyleClaimImpl(final String claimKey, final Function<Style, @Nullable V> lens, final Predicate<V> filter, final BiConsumer<V, TokenEmitter> emitable) {
      this.claimKey = claimKey;
      this.lens = lens;
      this.filter = filter;
      this.emitable = emitable;
   }

   public @NotNull String claimKey() {
      return this.claimKey;
   }

   public @Nullable Emitable apply(final @NotNull Style style) {
      V element = this.lens.apply(style);
      return element != null && this.filter.test(element) ? (emitter) -> {
         this.emitable.accept(element, emitter);
      } : null;
   }

   public int hashCode() {
      return Objects.hash(new Object[]{this.claimKey});
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof StyleClaimImpl)) {
         return false;
      } else {
         StyleClaimImpl<?> that = (StyleClaimImpl)other;
         return Objects.equals(this.claimKey, that.claimKey);
      }
   }
}
